// SourcePaige: 13-3-4-ASCIIDEFAULT-C/CPP

#include <windows.h>
#include <winsock.h>
#include <stdlib.h>

// Aaaaaagh!  Look for "!!!". (I compiler with the "warnings are treated as
// errors" switch *on*.

//===========================================================================

#ifndef CHAOS_UTILS_VERSION
#define CHAOS_UTILS_VERSION 0
#endif

//===========================================================================

typedef struct tagIPHDR
{
        u_char  version;
        u_char  tos;
        u_short len;
        u_short id;
        u_short off;
        u_char  ttl;
        u_char  p;
        u_short cksum;
        u_long  src;
        u_long  dst;
} IPHDR;

//===========================================================================

#define ICMP_ECHO               8
#define ICMP_ECHOREPLY          0
#define ICMP_UNREACH            3

typedef struct  tagICMPHDR
{
        u_char   type;
        u_char   code;
        u_short  cksum;
} ICMPHDR;

//===========================================================================

#define PINGDATASIZE    16

typedef struct tagPING
{
        IPHDR   ip;
        ICMPHDR icmp;
        u_short id;
        u_short seq;
        u_long  sendtime;
        u_char  data[ PINGDATASIZE ];
} PING;

//===========================================================================

u_long GetSysTimeMark( void )
{
        return (u_long)GetTickCount();
}

//===========================================================================

u_short in_cksum( u_short *addr, int len)
{
	int nleft = len;
	u_short *w = addr;
	int sum = 0;
	u_short answer = 0;

	/*
	 *  Our algorithm is simple, using a 32 bit accumulator (sum),
	 *  we add sequential 16 bit words to it, and at the end, fold
	 *  back all the carry bits from the top 16 bits into the lower
	 *  16 bits.
	 */
	while( nleft > 1 )  {
		sum += *w++;
		nleft -= 2;
	}

	/* mop up an odd byte, if necessary */
	if( nleft == 1 ) {
		*(u_char *)(&answer) = *(u_char *)w ;
		sum += answer;
	}

	/*
	 * add back carry outs from top 16 bits to low 16 bits
	 */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add possible carry */
	answer = ~sum;		/* ones complement & truncate to 16 bits */
	return (answer);
}

//===========================================================================

int ping( char *name, int timeoutms )
{
        SOCKET  s;
        SOCKADDR_IN     sai;
        struct protoent *proto;
        static int nsequence;
        u_long  destip;
        PING    p;
        int i;
        u_long t;

        destip = inet_addr( name );
        if ( destip == INADDR_NONE )
        {
                HOSTENT * host = gethostbyname( name );
                if ( host == NULL ) return -2;
                destip = *( (u_long *)(host->h_addr) );
        }

        proto = getprotobyname("icmp");

        s = socket( AF_INET, SOCK_RAW, proto->p_proto );

        sai.sin_family = AF_INET;
        sai.sin_addr.s_addr = destip;

        p.icmp.type     = ICMP_ECHO;
        p.icmp.code     = 0;
        p.icmp.cksum    = 0;

        p.id            = 100; // A "process ID"
        p.seq           = ++nsequence;
        p.sendtime      = GetSysTimeMark();

        for ( i = 0; i < PINGDATASIZE; i ++ )
                p.data[ i ] = 'A' + (i%26);

        p.icmp.cksum    =       in_cksum( (u_short *)&p.icmp.type,
sizeof(PING)-sizeof(IPHDR) );

        sendto( s, ((const char *)&p)+sizeof(IPHDR),
sizeof(PING)-sizeof(IPHDR), 0, (struct sockaddr *)&sai, sizeof(sai) );
        memset( &p, 0, sizeof(PING) );

        t = GetSysTimeMark();

        while ( (GetSysTimeMark()-t) < (u_long)timeoutms )      //!!!
        {
                u_long ar = 0;

                ioctlsocket(s, FIONREAD, &ar );
                if ( ar )
                {
                        SOCKADDR_IN sai_r;
                        int sai_sr = sizeof(SOCKADDR_IN);

                        char *incoming = (char *)&p;
                        int size = recvfrom( s, incoming, sizeof(p),0,
(struct sockaddr *)&sai_r, &sai_sr );

                        if ( size!=-1)
                        {
                                if ( p.ip.p == proto->p_proto )
                                {
                                        if ( sai_r.sin_addr.s_addr ==
sai.sin_addr.s_addr )
                                                if ( p.icmp.type ==
ICMP_ECHOREPLY )
                                                {
                                                        closesocket(s);
                                                        return (
GetSysTimeMark() - t );
                                                }
                                }
                        }
                }
        }

        closesocket(s);
        return -1;
}

//===========================================================================
#if     !CHAOS_UTILS_VERSION
//===========================================================================

char    g_pinghost[256] = { "127.0.0.1" };
int     g_pingtimeout = 100;
int     g_goodtime = 1;
int     g_period = 5000;
int     g_posx = 0;
int     g_posy = 0;
int     g_width = 100;
int     g_height = 24;

int     g_numdone = 0;
int     g_pingrc=-1;
UINT    g_timerid;

//===========================================================================

void ArgError( char *text )
{
        MessageBox( HWND_DESKTOP, text, "VPING: Argument Error", MB_OK );
        exit(0);
}

//===========================================================================

char *GetToken( char *s, char *dest )
{
        while ( (*s!='\0') && ( (*s==' ') || (*s=='\t') ) ) s++;
        if ( *s=='\0' ) return NULL;

        char    *rc = s;

        while ( (*s!='\0') && (*s!=' ') && (*s!='\t') ) s++;

        strncpy(dest, rc, s-rc );
        dest[s-rc] = '\0';
        return s;
}

void ProcessCommandLine( LPCTSTR szCommand )
{
        char    token[256];

        char    *s = (char *)szCommand;

        for (;;)
        {
                s = GetToken( s, token );
                if ( s == NULL ) return;

                if (!strcmp( token, "-host" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no string after -host"
);
                        strcpy( g_pinghost, token );
                }

                if (!strcmp( token, "-timeout" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -timeout"
);
                        g_pingtimeout = atoi( token );
                }

                if (!strcmp( token, "-good" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -good" );
                        g_goodtime = atoi( token );
                }

                if (!strcmp( token, "-period" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -period"
);
                        g_period = atoi( token );
                }

                if (!strcmp( token, "-posx" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -posx" );
                        g_posx = atoi( token );
                }

                if (!strcmp( token, "-posy" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -posy" );
                        g_posy = atoi( token );
                }

                if (!strcmp( token, "-width" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -width"
);
                        g_width = atoi( token );
                }

                if (!strcmp( token, "-height" ))
                {
                        s = GetToken( s, token );
                        if ( s == NULL ) ArgError( "no value after -height"
);
                        g_height = atoi( token );
                }
        }
}

//===========================================================================

void OnCreate( HWND hWnd )
{
        g_timerid = SetTimer( hWnd, 101, g_period, NULL );
}

void OnClose( HWND hWnd )
{
        KillTimer( hWnd, g_timerid );
        PostQuitMessage(0);
}

void OnTimer( HWND hWnd )
{
        KillTimer( hWnd, g_timerid );
        g_pingrc = ping( g_pinghost, g_pingtimeout );
        InvalidateRect( hWnd, NULL, true );
        g_timerid = SetTimer( hWnd, 101, g_period, NULL );
        g_numdone++;
}

void OnPaint( HWND hWnd )
{
        char runner[]={"-\\|/-\\|/"};

        HDC hdc;
        PAINTSTRUCT ps;

        hdc = BeginPaint( hWnd, &ps );

        RECT rect;

        GetClientRect( hWnd, &rect );

        COLORREF fill = RGB(255,255,0);
        if ( g_pingrc < g_goodtime ) fill = RGB(0,255,0);
        if ( g_pingrc < 0 ) fill = RGB(255,0,0);

        HPEN hpen = CreatePen( PS_SOLID, 1, fill );
        HBRUSH hbr = CreateSolidBrush( fill );

        HPEN   holdpen   =   (HPEN)SelectObject( hdc, hpen );
        HBRUSH holdbrush = (HBRUSH)SelectObject( hdc, hbr );

        Rectangle( hdc, rect.left, rect.top, rect.right, rect.bottom );

        SetBkMode( hdc, TRANSPARENT );
        SetTextColor( hdc, RGB(0,0,0));
        DrawText( hdc, g_pinghost, strlen(g_pinghost), &rect,
DT_CENTER|DT_VCENTER|DT_SINGLELINE );

        TextOut( hdc, 4,4, (const char *)&runner[ g_numdone%strlen(runner)
], 1 );

        SelectObject( hdc, holdpen );
        SelectObject( hdc, holdbrush );

        EndPaint( hWnd, &ps );
}

LRESULT CALLBACK wndprocframe( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
lParam )
{
        switch( uMsg )
        {
        case WM_CREATE: OnCreate( hWnd ); return 0;
        case WM_CLOSE:  OnClose( hWnd ); return 0;
        case WM_TIMER:  OnTimer( hWnd ); return 0;
        case WM_PAINT:  OnPaint( hWnd ); return 0;
        case WM_CHAR:
        case WM_LBUTTONDOWN:
                        SendMessage( hWnd, WM_CLOSE,0,0 ); return 0;

        default:        return DefWindowProc( hWnd, uMsg, wParam, lParam );
        }
}

void MessagePump( void )
{
        MSG msg;

        while ( msg.message!=WM_QUIT )
        {
                if ( PeekMessage( &msg, 0, 0 , 0, PM_REMOVE ) )
                {
                        TranslateMessage( &msg );
                        DispatchMessage( &msg );
                } else {
                        if ( !GetMessage( &msg, 0, 0, 0 ) ) return;
                        TranslateMessage(&msg);
                        DispatchMessage(&msg);
                }
        }
}

int WINAPI WinMain( HINSTANCE hInst, HINSTANCE hPInst, LPSTR lpszCommand,
int nShow )
{

//      FreeConsole();  // For some reason GCC apps have a console even if
                        // they are GUI. Get rid of it please!

        g_width = GetSystemMetrics( SM_CXFULLSCREEN );

        ProcessCommandLine( lpszCommand );

        WSADATA wsa;

        WSAStartup( 0x0101, &wsa );

        static char *cls = "VPING";

        WNDCLASS        wc;
        wc.style        = CS_HREDRAW | CS_VREDRAW;
        wc.lpfnWndProc  = &wndprocframe;
        wc.hInstance    = GetModuleHandle(NULL);
        wc.cbClsExtra   = 0;
        wc.cbWndExtra   = 0;
        wc.hIcon        = LoadIcon( NULL, IDI_APPLICATION );
        wc.hCursor      = LoadCursor( NULL, IDC_ARROW );
        wc.hbrBackground = NULL;
        wc.lpszMenuName   = NULL;
        wc.lpszClassName  = cls;
        RegisterClass( &wc );

        HWND hWndMain = CreateWindowEx( WS_EX_TOOLWINDOW|WS_EX_TOPMOST,
                                        cls,
                                        "Visual Ping ;-)",
                                        WS_POPUP | WS_BORDER,
                                        g_posx, g_posy,
                                        g_width, g_height,
                                        0,
                                        NULL,
                                        GetModuleHandle(NULL),
                                        0 );

        OnTimer( hWndMain );
        ShowWindow( hWndMain, nShow );

        MessagePump();

        return 0;
}

//===========================================================================
#endif  //CHAOS_UTILS_VERSION
//===========================================================================

